import 'package:code_id_storage/code_id_storage.dart';
import 'package:dio/dio.dart';

typedef AuthTokenBuilder<T> = Map<String, dynamic>? Function(T? token);

class AuthInterceptor<T> extends Interceptor {
  // final String? token;
  final IStorage<T> storage;
  final String authKey;
  final AuthTokenBuilder<T> authHeadersBuilder;

  // final String
  AuthInterceptor({
    required this.storage,
    required this.authKey,
    required this.authHeadersBuilder,
  });

  @override
  Future<void> onRequest(
    RequestOptions options,
    RequestInterceptorHandler handler,
  ) async {
    if (storage.isOpen()) {
      final T? token = storage.getData(key: authKey);

      final Map<String, dynamic>? authHeader = authHeadersBuilder(token);
      if (authHeader != null) {
        options.headers.addAll(authHeader);
      }
    }

    super.onRequest(options, handler);
  }
}
